import 'dart:async';
import 'dart:io';
import 'package:http/http.dart' as http;

final RegExp _header =
    new RegExp(r'^([A-Za-z]+((A-Za-z-)*[A-Za-z-]+)?):\s*([^\n]+)$');
final RegExp _responseStart =
    new RegExp(r'^HTTP/[0-9]+\.[0-9]+ ([0-9]+) ([^\n]+)$');

/// Sends HTTP requests through a rotating list of proxy endpoints.
class ProxyClient extends http.BaseClient {
  http.IOClient _inner;
  final HttpClient _innerClient = new HttpClient();
  final List<ProxyEndpoint> _proxies = [];
  final bool defaultToDirect;

  ProxyClient({Iterable<ProxyEndpoint> endpoints, this.defaultToDirect: true}) {
    this._proxies.addAll(endpoints ?? []);

    for (var proxy in _proxies) {
      if (proxy.auth != null)
        _innerClient.addProxyCredentials(
            proxy.host,
            proxy.port,
            proxy.auth.realm,
            new HttpClientBasicCredentials(
                proxy.auth.username, proxy.auth.password));
    }

    _innerClient.findProxy = (_) {
      if (_proxies.isEmpty) return 'DIRECT';

      var buf = new StringBuffer();

      for (int i = 0; i < _proxies.length; i++) {
        if (i > 0) buf.write('; ');

        var proxy = _proxies[i];
        buf.write('PROXY ${proxy.host}:${proxy.port}');
      }

      return buf.toString().trim() +
          (defaultToDirect == true ? '; DIRECT' : '');
    };

    _inner = new http.IOClient(_innerClient);
  }

  @override
  void close() => _inner.close();

  @override
  Future<http.StreamedResponse> send(http.BaseRequest request) {
    return _inner.send(request);
  }
}

class ProxyEndpoint {
  final String host;
  final ProxyEndpointAuth auth;
  final int port;
  final bool ssl;

  ProxyEndpoint({this.host, this.auth, this.port, this.ssl});
}

class ProxyEndpointAuth {
  String username, password, realm;
  ProxyEndpointAuth({this.username, this.password, this.realm});
}
